'use strict'

/**
 * Module dependencies.
 */

const { chalk } = require('@vuepress/shared-utils')
const envinfo = require('envinfo')

const { dev, build, eject } = require('../core')
const { path, logger, env } = require('@vuepress/shared-utils')
const { wrapCommand } = require('./util')

// const { join } = require('path')
// const { red, yellow, gray } = chalk
// const puppeteer = require('puppeteer')
// const PDFMerge = require('easy-pdf-merge')

/**
 * Expose registerCoreCommands function.
 */

module.exports = function(cli, options) {
  cli
    .command(`dev [targetDir]`, 'start development server')
    .option('-p, --port <port>', 'use specified port (default: 8080)')
    .option('-t, --temp <temp>', 'set the directory of the temporary file')
    .option('-c, --cache [cache]', 'set the directory of cache')
    .option('--host <host>', 'use specified host (default: 0.0.0.0)')
    .option('--no-cache', 'clean the cache before build')
    .option('--no-clear-screen', 'do not clear screen when dev server is ready')
    .option('--debug', 'start development server in debug mode')
    .option('--silent', 'start development server in silent mode')
    .option('--open', 'open browser when ready')
    .action((sourceDir = '.', commandOptions) => {
      const { debug, silent } = commandOptions

      logger.setOptions({ logLevel: silent ? 1 : debug ? 4 : 3 })
      logger.debug('global_options', options)
      logger.debug('dev_options', commandOptions)
      env.setOptions({ isDebug: debug, isTest: process.env.NODE_ENV === 'test' })

      wrapCommand(dev)({
        sourceDir: path.resolve(sourceDir),
        ...options,
        ...commandOptions
      })
    })

  cli
    .command('build [targetDir]', 'build dir as static site')
    .option('-d, --dest <dest>', 'specify build output dir (default: .vuepress/dist)')
    .option('-t, --temp <temp>', 'set the directory of the temporary file')
    .option('-c, --cache [cache]', 'set the directory of cache')
    .option('--dest <dest>', 'the output directory for build process')
    .option('--no-cache', 'clean the cache before build')
    .option('--debug', 'build in development mode for debugging')
    .option('--silent', 'build static site in silent mode')
    .option('--max-concurrency', 'set the max docs concurrently processed when build static site')
    .action((sourceDir = '.', commandOptions) => {
      const { debug, silent } = commandOptions

      logger.setOptions({ logLevel: silent ? 1 : debug ? 4 : 3 })
      logger.debug('global_options', options)
      logger.debug('build_options', commandOptions)
      env.setOptions({ isDebug: debug, isTest: process.env.NODE_ENV === 'test' })

      wrapCommand(build)({
        sourceDir: path.resolve(sourceDir),
        ...options,
        ...commandOptions
      })
    })

  cli
    .command('eject [targetDir]', 'copy the default theme into .vuepress/theme for customization.')
    .option('--debug', 'eject in debug mode')
    .action((dir = '.') => {
      wrapCommand(eject)(path.resolve(dir))
    })

  cli
    .command('info', 'Shows debugging information about the local environment')
    .action(() => {
      console.log(chalk.bold('\nEnvironment Info:'))
      envinfo.run(
        {
          System: ['OS', 'CPU'],
          Binaries: ['Node', 'Yarn', 'npm'],
          Browsers: ['Chrome', 'Edge', 'Firefox', 'Safari'],
          npmGlobalPackages: ['vuepress'],
          npmPackages: [
            'vuepress',
            '@vuepress/core',
            '@vuepress/theme-default'
          ]
        },
        {
          showNotFound: true,
          duplicates: true,
          fullTree: true
        }
      ).then(console.log)
    })

//   cli
//     .command('exportPDF', 'export current vuepress site to a PDF file')
//     .allowUnknownOptions()
//     .action(async(dir = '.', commandOptions) => {
//       try {
//         const nCtx = await dev({
//           sourceDir: path.resolve('.'),
//           clearScreen: false,
//           // theme: opts.theme || '@vuepress/default'
//         })
//         logger.setOptions({ logLevel: 3 })
//         logger.info(`Start to generate current site to PDF ...`)
//         try {
//           await generatePDF(nCtx, nCtx.devProcess.port, nCtx.devProcess.host)
//         } catch (error) {
//           console.error(red(error))
//         }
//         nCtx.devProcess.server.close()
//         process.exit(0)
//       } catch (e) {
//         throw e
//       }
//     })
}
//
// async function generatePDF(ctx, port, host) {
//   const { pages, tempPath, siteConfig } = ctx
//   const tempDir = join(tempPath, 'pdf')
//   fs.ensureDirSync(tempDir)
//
//   const exportPages = pages.map(page => {
//     return {
//       url: page.path,
//       title: page.title,
//       location: `http://${host}:${port}${page.path}`,
//       path: `${tempDir}/${page.key}.pdf`
//     }
//   })
//
//   const browser = await puppeteer.launch()
//   const browserPage = await browser.newPage()
//
//   for (let i = 0; i < exportPages.length; i++) {
//     const {
//       location,
//       path: pagePath,
//       url,
//       title
//     } = exportPages[i]
//
//     await browserPage.goto(
//       location,
//       { waitUntil: 'networkidle2' }
//     )
//
//     await browserPage.pdf({
//       path: pagePath,
//       format: 'A4'
//     })
//
//     logger.success(
//       `Generated ${yellow(title)} ${gray(`${url}`)}`
//     )
//   }
//
//   const files = exportPages.map(({ path }) => path)
//   const outputFilename = siteConfig.title || 'site'
//   const outputFile = `${outputFilename}.pdf`
//   await new Promise(resolve => {
//     PDFMerge(files, outputFile, err => {
//       if (err) {
//         throw err
//       }
//       logger.success(`Export ${yellow(outputFile)} file!`)
//       resolve()
//     })
//   })
//
//   await browser.close()
//   fs.removeSync(tempDir)
// }
